/**
 *	Copyright (C) 2011-2016 Docuteam GmbH
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 3
 *	as published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.docuteam.aipcreatorETH.admin;


/**
 * @author denis
 *
 * CHANGES HISTORY:
 *
 * Version 1.3.8
 * 31.01.2017:
 * - handle migrated complex objects as IE with MODIFIED and PRESERVATION
 * 26.01.2017:
 * - bump to next development version
 *
 * Version 1.3.7
 * 25.01.2017:
 * - handle migrated simple objects as IE with MODIFIED and PRESERVATION
 * - update dependency to docuteam-darc-2.18.5
 * - update dependency to docuteam-tools-1.11.8
 * 24.01.2017:
 * - fix sql script
 * 19.01.2017:
 * - update dependency to dps-sdk-5.1.0 (requires commons-lang-2.3)
 * 17.01.2017:
 * - bump to next development version
 *
 * Version 1.3.6
 * 17.01.2017:
 * - fix inserts of already existing collection DB entries (matching DB unique index IDX_U03_ING_TRACK_REC_FEEDER)
 * 20.12.2016:
 * - bump to next development version
 *
 * Version 1.3.5
 * 20.12.2016:
 * - update dependency to docuteam-darc-2.18.4
 * - don't create a DB entry for collection entries that have been created with a previous partial submit.
 * 16.11.2016:
 * - bump to next development version
 *
 * Version 1.3.4
 * 16.11.2016:
 * - fix deletion of DB entries to be node specific (and note entire SIPs) in case of test runs
 * 15.11.2016:
 * - fix handling of SIP names containing dots
 * 10.11.2016:
 * - bump to next development version
 *
 * Version 1.3.3
 * 09.11.2016:
 * - fix submit status in EAD stream of Rosetta IEs
 * 02.11.2016:
 * - bump to next development version
 *
 * Version 1.3.2
 * 02.11.2016:
 * - remove property AIPCreator.IngestFeedbackDir again (relative to the submitted SIPs' directory as both packer and feeder need to communicate through it)
 * 31.10.2016:
 * - bump to next development version
 *
 * Version 1.3.1
 * 31.10.2016:
 * - include processing of characteristics/characteristicsNote for website crawls
 * 28.10.2016:
 * - bump to next development version
 *
 * Version 1.3.0
 * 28.10.2016:
 * - update dependency to docuteam-darc-2.18.1
 * - fix SSH exception message
 * - fix exception handling on submit
 * - fix replacing of mets.xml from feedback
 * 27.10.2016:
 * - have all properties start with lowercase characters (default in all other packages)
 * - extend SSH mode for submit and checkIngestFeedback
 * - update dependency to docuteam-darc-2.18.0
 * - remove check for docuteam-darc library version
 * - add mandatory property AIPCreator.IngestFeedbackDir
 * - check properties depending on method context
 * - transfer levels.xml during submit with SSH as well
 * - move checkIngestFeedback from mets.Document to AIPCreatorETH
 * - cleanup submit methods
 * - bump to next minor release (change of interface ch.docuteam.darc.ingest.AIPCreator)
 * 09.05.2016:
 * - admin.Version: bump to next development version
 *
 * Version 1.2.0
 * 09.05.2016 (Andreas):
 * - lib/docuteam-darc-2.16.1.jar: updated version
 * 		- mets.structmap.NodeFile: modify checksum calculation to work if submitStatus is not used
 * - ingest.AIPCreatorETH:
 * 		- add DOI to DB table
 * 		- change field "FEEDER_PATH" into "SOURCE_PATH"
 * - resources/sql/*_ingest_tracking_record_feeder.sql:
 * 		- add field for DOI
 * 		- change field "FEEDER_PATH" into "SOURCE_PATH"
 * 04.05.2016 (Andreas):
 * - lib/docuteam-darc-2.16.0.jar: updated version
 * 		- partial submit, optionally through SSH
 * 		- migration keeping original with subfolder
 * - lib/docuteam-tools-1.11.2.jar: update to current version, changes since version 1.11.1:
 * 		- file.ssh.SFTPFileTransfer, file.ssh.SSHSessionProvider: new classes for SFTP file transfer
 * 		- lib/jsch-0.1.53.jar: add java ssh library
 * - admin.Version: bump to next minor version
 * 07.04.2016 (Andreas):
 * - lib/docuteam-darc-2.15.0.jar: updated version
 * 		- mdconfig.LevelMetadataElement, mdconfig.MetadataElement, mets.structmap.NodeAbstract: validate default expressions against metadata validators and allowed values
 * 		- premis.Object: set default algorithm to SHA-512
 * 		- mets.structmap.NodeFile:
 * 			- instead of hard-coded use of MD5, select the specified algorithm to check fixity.
 * 			- verify checksums only for submitted nodes.
 * - lib/docuteam-tools-1.11.1.jar: update to current version, changes since version 1.11.0:
 * 		- translations.I18N: extended to load several resource bundles per language
 * 		- config_dist/container-signature-20160121.xml, config_dist/DROID_SignatureFile_V84.xml, file.MetadataProviderDROID: update to current DROID signature files
 * - lib/docuteam-tools-1.11.0: use tagged version:
 * 		- file.FileChecksumCalculator, file.FileWithMetadata: add checksum calculations for SHA-256 and SHA-512
 * 21.03.2016 (Christian):
 * *.java: update copyright remarks
 * 19.02.2016 (Andreas):
 * - lib/docuteam-darc-2.15.0.jar: updated version
 * 		- merged tags 2.14.4 to 2.14.10 from trunk:
 * 			- ingest.AIPCreatorBasic: new AIPCreator that only checks for mandatory metadata elements and readability of files
 * 			- config_levels/levels_all_*.xml: include additional icon filenames
 * 			- mets.Document: simplify filter for valid SIP folders
 * 			- mets.Document: add method saveAndClose which moves the sip instead of copying it (closing the working copy and setting it into ReadWriteNoFileOps mode) using less resources
 * 			- mets.Document, mdconfig.LevelOfDescription: use SAXReader.read() with a File instead of a String to handle file paths with special characters
 * 			- mets.amdsec.DigiprovWithPremis: new method signatures to create PREMIS events with a specific date
 * 			- mdconfig.MetadataElementValidatorDate*: accept a value of 'NoDate' (and translations if available, e.g. "undatiert")
 * 			- mdconfig.MetadataElementValidator*CH: allow dates to have single digits for month/day components
 * 			- mdconfig.MetadataElementValidatorDateHierarchyRangeCH: fix check for null values in child nodes
 * 			- mdconfig.MetadataElementValidatorDateHierarchyRangeCH: fix to include the "NoDate" string into interpretation
 * 			- mdconfig.MetadataElementValidatorDateHierarchyYear: fix to deal with empty/null values and check against child nodes as well
 * 			- mdconfig.MetadataElementValidatorDateRangeCH: fix to allow dates with single digit days/months
 * 		- docuteam-tools-1.10.9.jar
 * 16.02.2016 (Andreas):
 * - ingest.AIPCreatorETH:
 * 		- generateAIP(): allow re-submit for Collection nodes
 * 		- confirmStorage(): add Rosetta deep link to metadata (additional properties "AIPCreator.DeeplinkPatternCollection" and "AIPCreator.DeeplinkPatternIE")
 * - unittests:
 * 		- extend test for generateAIP (partial submits)
 * 		- new test for confirmStorage
 * - lib/docuteam-darc-2.15.0.jar: updated version
 * 		- mets.Document:
 * 			- submit will re-submit parent nodes of selected nodes
 * 			- automatically deleteOnExit working copies of zipped SIP files that were opened read-only
 * 			- submit only transfers the files that are selected for submit, not the entire SIP anymore
 * 			- fix method getSIPNameForSubmit() to allow for dots in SIP names.
 * 		- mets.structmap.NodeAbstract:
 * 			- setDynamicMetadataValueForName_force() will ignore SubmitStatus (allowing to set metadata during ingest for submitted nodes)
 * 			- When setting the SubmitStatus recursively, include the parent hierarchy as well
 * 08.02.2016 (Andreas):
 * - ingest.AIPCreatorETH: adapt to migration keeping original as preservation master and preservation copy
 * - test.*: use local MySQL instance instead of remote Oracle
 * 08.09.2015 (Andreas):
 * - ingest.AIPCreatorETH: set submitStatus prospectively in EAD for Rosetta objects
 * - lib/docuteam-darc-2.15.0.jar:
 * 		- mets.Document:
 * 			- add method hasSubmittedNodes() to check if parts of the SIP have been submitted.
 * 			- properly lock SIPs while being submitted
 * 			- properly unlock SIPs after getting the feedback
 * 		- structMap.nodeFile: fix migration when keeping original, take into account that the SIP can have a submit status blocking regular modifications
 * 		- util.XMLUtil: replace both \r and \n with <EAD:lb> elements
 * - lib/docuteam-tools-1.10.8: Use final tagged version
 * 		- file.FileUtil: fix basicCopyTo method to ignore copying if source equals destination
 * - lib/docuteam-tools-1.10.7: Use final tagged version
 * 		- lib/log4j-1.2.17.jar: updated version
 * 		- lib/commons-codec-1.9.jar: dependency of poi-3.11-20141221.jar
 * 10.08.2015 (Andreas):
 * - ingest.AIPCreatorETH: don't check nodes that are not to be submitted
 * - lib/docuteam-darc-2.15.0.jar:
 * 		- mets.structMap.NodeAbstract: split method setSubmitStatusRecursivelyAllOrNone() into two separate ones: ..._check() and _force()
 * 
 * Version 1.1.0
 * 07.07.2015 (Andreas):
 * - lib/docuteam-darc-2.15.0.jar:
 * 		- mets.Document: remove locking of entire SIPs after submit and submit feedback
 * 06.07.2015 (Andreas):
 * - ingest.AIPCreatorETH: adjust various methods to allow specific selection of nodes to submit
 * - lib/docuteam-darc-2.15.0.jar:
 * 		- mets.structmap.NodeFile: change behavior when migrating files to create a new subfolder containing original and copy
 * 		- mets.structMap.NodeAbstract: add method to switch a specific SubmitStatus for all nodes from one state to another
 * 		- mets.Document: use the above mentioned new method to prepare the SIP for submission
 * - admin.Version:
 * 		- bump to next minor version
 * 		- increase required darc version
 * 28.04.2015 (Andreas):
 * - admin.Version: bump to next version
 *
 * Version 1.0.0
 * 28.04.2015 (Andreas):
 * - admin.Version: bump to version 1.0.0
 * - config/*: removed deprecated DROID signature files
 * - unittests.ingest.AIPCreatorETH: adapt to java 8, take into account that a specific order within a HashMap might not be guaranteed.
 * 15.04.2015 (Andreas):
 * - admin.Version: increase required darc version
 * - lib/docuteam-darc-2.14.2.jar: use tagged library version with
 * 		- additional metadata elements
 * 		- revised handling of PREMIS events
 * 		- resources/templates/mets_sip.xml: remove <metsHdr> to enable initialization of a clean header for new SIPs.
 * 		- mets.amdsec.DigiprovWithPremis: updated PREMIS_Version to 2.2
 * 		- mets.structmap.NodeAbstract: add exception for root file if mimetype or PUID is unknown
 * 		- mets.Document: change xml output format to be compact; saves space, formatting wasn't really indented in a readable way before either.
 * 		- mdconfig.MetadataElementValidatorDateHierarchyRangeCH: ignore children if respective MetadataElement is not available with them
 * 		- mets.Document: collect exception if mets file has no TYPE attribute (i.e. reference for a submission agreement)
 * - lib/docuteam-tools-1.10.6.jar: use tagged library version
 * 		- config/*: updated DROID signature files
 * 		- string.StringUtil: new method isDecimal(String)
 * 		- lib/*: updated droid and poi libraries
 *
 * Version 1.0.0i
 * 17.02.2015 (Andreas):
 * build.xml: set java compatibility to 1.7
 * lib/docuteam-tools-1.10.6.jar: use java 1.7 compatible build
 * 09.02.2015 (Christian):
 * lib/docuteam-darc: new version 2.14.2 (now really)
 * 09.02.2015 (Andreas):
 * build.xml: fix manifest classpath
 * 03.02.2015 (Christian):
 * lib/docuteam-darc: new version 2.14.2:
 * 		- additional metadata elements: deathOfAuthor, accessionNumber, sourceType, usagePermission, usagePermissionExpiringDate, modeOfAcquisition, digitization, relationPeriod
 * lib/docuteam-tools: new version 1.10.5 with updated libraries:
 * 		- lib/droid*.jar to 6.1.5
 * 		- lib/poi*.jar to 3.11
 * build.xml updated to java version 1.8
 * 14.01.2015 (Andreas):
 * ingest.AIPCreatorETH: use the regular slash in general as file separator in metadata
 * 14.01.2015 (Christian):
 * unittest.aipcreatorETH.ingest.AIPCreatorETH: changed file separator in assert to meet system dependent separator.
 * 17.11.2014 (Andreas):
 * test._1_CreateAIP: use test server
 * resources/sql/*: create scripts for oracle database
 * build.xml: include support files in dist
 * 13.11.2014 (Andreas):
 * Use tagged version of darc 2.14.1 with tagged tools 1.10.4
 * unittest.ingest.AIPCreatorETH: use oracle db on test server
 * unittest/files/SIPs/SubmitTest_OK_Submitted.zip: remove line break encoding problems
 * 12.11.2014 (Andreas):
 * licenses/*: cleanup
 * ingest.AIPCreatorETH: use prepared statements instead of plain text statements to prevent sql injection and do automatic character escaping.
 * 11.11.2014 (Andreas):
 * ingest.AIPCreatorETH: add workspace label to the file-IE's <dnx:key @id="fileOriginalPath"> (cf. ETH-166)
 * 
 * Version 1.0.0h
 * 05.09.2014 (Andreas):
 * config/*: remove converter configuration files as they are not needed in this package
 * Use current version of darc 2.14.1 with tools 1.10.4
 * licenses/*: general cleanup
 * 
 * Version 1.0.0g
 * 28.08.2014 (Andreas):
 * Use tagged version of darc 2.14.0 with extension:
 * 		use current version of tools 1.10.3 with fix when copying files
 * 		mdconfig.MetadataElement: Validate every new value against allowedValues, too (if defined). Tons of resulting changes.
 * 		mdconfig.MetadataElementValidatorUniqueValueAmongSiblings: Correction: Check if node to test is root and hence has no siblings.
 * 		mdconfig.*: Validators now return a message code which should be translated.
 * unittest.aipcreatorETH.ingest.AIPCreatorETH: adopt test to recent change regarding <dcterms:isPartOf>
 * 20.06.2014 (Denis):
 * Use current version of darc 2.14.0 with extension:
 * 		translations.I18N: Add methods translate_NoCheck(String word) and translate_NoCheck(String word, Object... objects) to avoid exception when I18N is not initialized.
 * 		mdconfig.MetadataElementValidatorUniqueValueAmongSiblings: Translate metadata name in warning message.
 * 		mdconfig.MetadataElementValidatorDate, mdconfig.MetadataElementValidatorDateCH: Correction in parsing dates: be more strict.
 * ingest.AIPCreatorETH: Correction: In Collection-Files, insert at top-level <dcterms:isPartOf> with content: "workspace"
 * 		(before, <dcterms:isPartOf> was not set at top-level).
 * 17.06.2014 (Denis):
 * Use current version of darc 2.14.0 with corrections and extensions:
 * 		mdconfig.MetadataElementSetterPostAction, mdconfig.MetadataElement, mdconfig.LevelMetadataElement: Corrections and UnitTests.
 * 		Interface change:
 * 		mdconfig.MetadataElementValidator:
 * 				The method check(...) doesn't return a String any more but now throws a MetadataElementValidatorException.
 * 				Consequent changes in many other places.
 * 		mdconfig.MetadataElementSetterPostAction, mdconfig.LevelOfDescription, mdconfig.MetadataElement, mdconfig.LevelMetadataElement,
 * 		exceptions.MetadataElementSetterPostActionClassNotFoundException, config_dist/levels.xsd:
 * 				Execute an optional configurable post-action after having set a dynamic metadata value.
 * 13.06.2014 (Denis):
 * Use current version of darc 2.13.2 with correction:
 * mets.structmap.NodeAbstract, mdconfig.MetadataElementInstance, mdconfig.LevelMetadataElement, exceptions.LevelMetadataElementIsReadOnly:
 * 		The normal setter for dynamic metadata now throws the exception LevelMetadataElementIsReadOnly when the corresponding metadata
 * 		element is read-only for this level. The setter with an appended "_force" sets the value ignoring any read-only.
 * 12.06.2014 (Denis):
 * Use current version of darc 2.13.1 with corrections and extensions:
 * 		util.CSVExport: New class.
 * 		util.XML2PDF: set proper output, use saxon as xsl processing engine.
 * 		lib/: added current libraries for XSL-FO processing
 * 		mets.Document, mets.structmap.NodeAbstract: Improve the searchForAllQuoted method: ALL search words must be present, but each in any of the searched areas.
 * 04.06.2014 (Denis):
 * Use current version of darc 2.13.1 with corrections and extensions:
 * 		MetadataProviderDROID: Use full extension matching
 * 		ch.docuteam.darc.mdconfig.MetadataElementValidatorDateCH added. Checks if a date conforms to swiss dateformat.
 * 		ch.docuteam.unittest.darc.mdconfig.MetadataElementValidatorDateCH for unittest.
 * 		ch.docuteam.darc.mdconfig.MetadataElementValidatorUniqueValueAmongSiblings added. Checks if a refCode within same level is unique.
 * 		ch.docuteam.unittest.darc.mdconfig.MetadataElementValidatorUniqueValueAmongSiblings for unittest.
 * AIPCreatorETH: Correction: Catch and ignore exception "MetadataElementIsNotDefinedException" when reading retentionPolicy, accessRestrictionPeriod, and retentionPeriod.
 * AIPCreatorETH: Correction: Add the metadata "dc:title" and "dcterms:isPartOf" to the Collection entity.
 * 29.05.2014 (Andreas):
 * ingest.AIPCreatorETH: bugfix to only remove extension when SIP is zipped.
 * 28.05.2014 (Denis):
 * Use current version of darc 2.13.1 with extensions:
 * 		mets.Document, mets.structmap.NodeAbstract: Extend search to multiple search items, quoted.
 * 		mets.Document, exceptions.ZIPDoesNotContainMETSFileException: When a zipped SIP doesn't contain the METS file at the root level, throw this exception.
 * 22.05.2014 (Denis):
 * Use current version of darc 2.13.1 with correction:
 * 		mets.Document: Corrections in methods submit(...) and checkIngestFeedback(...): can now handle correctly both zipped and unzipped SIPs.
 * ingest.AIPCreatorETH: Corrections: can now handle correctly both zipped and unzipped SIPs.
 * 15.05.2014 (Denis):
 * AIPCreatorETH: Corrections in mapping of metadata when generating IEs and Collections.
 * 14.05.2014 (Denis):
 * Use current version of darc 2.13.1.
 * 13.05.2014 (Denis):
 * AIPCreatorETH$DB: Refactoring: Class is now static abstract. We use now two tables in the DB: "ingest_tracking_record_feeder" and "code_mapping".
 * Don't use class KeyAndValue to get rosetta keys, but AIPCreatorETH$DB.getRosettaCode(...), which retrieves the code from the DB table "code_mapping".
 * 12.05.2014 (Denis):
 * Use current version of darc 2.13.1.
 * 29.04.2014 (Andreas):
 * ingest.AIPCreatorETH: extend with new exceptions thrown by darc 2.13.0
 * config_dist: remove unnecessary files originally related to docudarc.FileConverter
 * Use current version of darc 2.13.0:
 * 		BAR-SIP-Writer
 * 		Additional exceptions
 * 		Additional metadata elements
 * Use current version of tools 1.10.1:
 * 		updated DROID signature file
 * 26.03.2014 (Denis):
 * Use current version of darc 2.12.0.
 * Use current version of tools 1.10.0.
 * 17.03.2014 (Denis):
 * Use current version of darc 2.12.0.
 * Use current version of tools 1.10.0.
 * Get rid of unused jars.
 * 17.02.2014 (Denis):
 * Use current version of darc 2.12.0.
 * Use current version of tools 1.10.0.
 * 10.02.2014 (Denis):
 * Use current version of darc 2.12.0.
 * Use current version of tools 1.10.0.
 * 04.02.2014 (Denis):
 * Use current version of Docudarc 2.11.4 with corrections:
 * 		mets.Document: Method "checkIngestFeedback()": Proper cleanup after feedback, including removal of the submitted zip file.
 * ingest.AIPCreatorETH: Fixed a bug where the files within a complex IE remained at the status "Pending".
 * 31.01.2014 (Denis):
 * Use current version of Docudarc 2.11.4 with corrections:
 * 		mets.Document:
 * 				Include in "checkSubmission()" the test if the packet already exists in the submission folder.
 * 				Make "checkIngestFeedback()" more strict, and check zipped SIPs as well.
 * 30.01.2014 (Denis):
 * ingest.AIPCreatorETH: Small bug fixes in file handling.
 * op: Refactoring: Renamed to "test".
 * ingest.AIPCreatorETH: Fixed a bug where the files within a complex IE remained at the status "Pending".
 * 29.01.2014 (Denis):
 * ingest.AIPCreatorETH: Fixed a bug where the files within a complex IE remained at the status "Pending".
 * 27.01.2014 (Denis):
 * ingest.AIPCreatorETH: Bug fix when writing AIP collection: set now charset of "UTF-8".
 * 24.01.2014 (Denis):
 * ingest.AIPCreatorETH: Small improvements in error messaging.
 * Removed folder "temp" from SVN.
 * 23.01.2014 (Denis):
 * Use current version of Docudarc 2.11.4 with correction:
 * 		mets.Document:
 * 				When submitting, zip the SIP without the enclosing folder.
 * 				Made method "boolean isZIPFile()" public.
 * ingest.AIPCreatorETH: When creating an AIP and the SIP is a zip, remove the '.zip' from the AIP name.
 * 22.01.2014 (Denis):
 * ingest.AIPCreatorETH: Whether field "PID" is optional or not, is now defined in levels.xml.
 * 16.01.2014 (Denis):
 * Use current version of Docudarc 2.11.4.
 * All java sources: Change copyright message from "2011-2013" to "2011-2014".
 * 15.01.2014 (Denis):
 * op.*: Corrections in test classes _1_CreateAIP, _2_IngestIntoRosetta, and _3_CreateFeedback: adapt to additional oracle db connection parameters.
 * 14.01.2014 (Denis):
 * Use current version of Docudarc 2.11.4.
 * 12.01.2014 (Andi):
 * ingest.AIPCreatorETH: Add additional oracle db connection parameter (database name); previously, the database name was in fact the service name.
 * Use current version of Docudarc 2.11.4.
 * 08.01.2014 (Denis):
 * ingest.AIPCreatorETH: Corrections creating Collections.
 * 07.01.2014 (Denis):
 * ingest.AIPCreatorETH:
 * 		Correction in error messages.
 * 		Renamed submitPath to aipPath.
 * 		Removed the property "feedbackFolder" as it is not needed any more.
 * 20.12.2013 (Denis):
 * ingest.AIPCreatorETH$DB: Correction: When no DB is configured, method wasAlreadyIngested() must return false, not true.
 * Use current version of Docudarc 2.11.4.
 * 13.12.2013 (Denis):
 * ingest.AIPCreatorETH: Feedback.
 * 12.12.2013 (Denis):
 * op._1_CreateAIP, op._2_IngestIntoRosetta, op._3_CreateFeedback: Distinct operations for the different workflow steps.
 * 11.12.2013 (Denis):
 * ingest.AIPCreatorETH: Feedback.
 * 10.12.2013 (Denis):
 * ingest.AIPCreatorETH:
 * 		Write into DB while creating AIP (corrections).
 * 		Feedback.
 * 09.12.2013 (Denis):
 * build.xml: Include lib/ojdbc6.jar in target 'dist'.
 * 03.12.2013 (Denis):
 * lib/ojdbc6.jar: Oracle jdbc driver.
 * ingest.AIPCreatorETH: Write into DB while creating AIP.
 * 02.12.2013 (Denis):
 * Use Docudarc 2.11.4 with correction:
 * 		ingest.AIPCreatorProxy: Don't thow exception when initialized with null or empty String.
 * 29.11.2013 (Denis):
 * ingest.AIPCreatorETH: Mapping of retentionPolicy code and retentionPeriod code.
 * 28.11.2013 (Denis):
 * ingest.AIPCreatorETH: Corrections in mapping requested by ETH.
 * 27.11.2013 (Denis):
 * Use Docudarc 2.11.4 with extension:
 * 		mdconfig.MetadataElement: allowedValues is now a List<KeyAndValue> and not any more a List<String>.
 * ingest.AIPCreatorETH: Corrections creating Collections.
 * 26.11.2013 (Denis):
 * ingest.AIPCreatorETH: Refuse to submit or submitCheck if the found Docudarc version is not equal to the RequiredDocudarcVersionNumber.
 * ingest.AIPCreatorETH: Transfer more dynamic metadata to the DC element when creating the AIP.
 * 24.11.2013 (Denis):
 * ingest.AIPCreatorETH: Check if the existing Docudarc version matches the required Docudarc version.
 * build.xml: On dist, don't include all required libraries any more but only AIPCreatorETH.jar.
 * 22.11.2013 (Denis):
 * ingest.AIPCreatorETH: Ignore null and empty values when transferring metadata to IEs and Collections.
 * 21.11.2013 (Denis):
 * ingest.AIPCreatorETH: New class.
 *
 * Version 0.0.0:
 * 21.11.2013 (Denis):
 * Initialization and setup.
 *
 */
public abstract class Version
{
	static public final String Product = "AIPCreatorETH";
	static public final String Number = "1.3.8";
	static public final String LastChange = "31.01.2017";

	/**
	 * This main method is being used by the ant build script to be able to add a version number to the compiled jar file.
	 *
	 * @param args: No parameters are expected.
	 */
	public static void main(String[] args)
	{
		System.out.println(Number);
	}
}
