/**
 *	Copyright (C) 2011-2016 Docuteam GmbH
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 3
 *	as published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.docuteam.aipcreatorETH.test;

import java.util.Properties;

import ch.docuteam.aipcreatorETH.ingest.AIPCreatorETH;
import ch.docuteam.darc.ingest.AIPCreatorProxy;
import ch.docuteam.darc.mets.Document;
import ch.docuteam.tools.os.OperatingSystem;
import ch.docuteam.tools.out.Tracer;
import ch.docuteam.tools.string.DateFormatter;

/**
 * @author denis
 *
 */
public abstract class _1_CreateAIP
{
	//	===========================================================================================
	//	========	Main					=======================================================
	//	===========================================================================================

	/**
	 * @param args
	 * @throws Exception
	 */
	public static void main(String[] args) throws Exception
	{
		if (args.length == 0)		throw new NullPointerException("No SIP Name and isTestMode-flag specified");
		if (args.length == 1)		throw new NullPointerException("No isTestMode-flag specified");
		String sipName = args[0];
		if (sipName.isEmpty())		throw new NullPointerException("Empty SIP Name specified");

		//	If this is true, it makes the test repeatable by deleting the destination AIP and clearing some rows in the DB:
		AIPCreatorETH.IsTestRun = new Boolean(args[1]);

		String feederFolder = OperatingSystem.userHome() + "Desktop/Workspace/Standard_DOI";
		String ingestDropFolder = feederFolder + "/dropbox";
		String aipFolder = feederFolder + "/AIPs";

		//	Set up the AIPCreatorProxy:
		AIPCreatorProxy.initializeImpl(new AIPCreatorETH());
		Properties props = new Properties();
		props.setProperty("AIPCreator.aipFolderPath", aipFolder);
		props.setProperty("AIPCreator.executionId", DateFormatter.getCurrentDateTimeString("hhmmssSSS"));
		props.setProperty("AIPCreator.dbHost", "jdbc:mysql://localhost");
		props.setProperty("AIPCreator.dbPort", "3306");
		props.setProperty("AIPCreator.dbService", "eth_ingest");
		props.setProperty("AIPCreator.dbName", "eth_ingest");
		props.setProperty("AIPCreator.dbUserName", "unittest");
		props.setProperty("AIPCreator.dbUserPassword", "docudemo");
		props.setProperty("AIPCreator.deeplinkPatternCollection", "http://rossa-test-ap11.ethz.ch:1801/delivery/action/collectionViewer.do?operation=viewCollection&collectionId=%s");
		props.setProperty("AIPCreator.deeplinkPatternIE", "http://rossa-test-ap11.ethz.ch:1801/delivery/DeliveryManagerServlet?dps_pid=%s");
		AIPCreatorProxy.initialize(props);

		//	Generate AIP: Create IEs and Collections from the nodes marked as "submitRequestPending":
		Document doc = Document.openReadOnly(ingestDropFolder + "/" + sipName, "Document.main");
		Tracer.trace(AIPCreatorProxy.generateAIP(doc));

		System.exit(0);
	}

}
