/**
 *	Copyright (C) 2011-2016 Docuteam GmbH
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 3
 *	as published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.docuteam.packerETH.admin;


/**
 * @author denis
 *
 * CHANGES HISTORY:
 *
 * Version 2.4.11a
 * 31.01.2017:
 * update dependency to DocuteamPacker 2.4.11:
 * 		- property for migration default
 * 		- fix NullPointerException
 * update dependency for aipcreatorETH to 1.3.7
 * 		- complex IE with migrated files
 * 30.01.2017:
 * re-label EAD export template for CMISTAR
 * 26.01.2017:
 * bump to next bugfix release
 * 
 * Version 2.4.10a
 * 26.01.2017:
 * update dependency to DocuteamPacker 2.4.10:
 * 		- migration action
 * 		- checksum validation action
 * update dependency for aipcreatorETH to 1.3.7
 * 16.12.2016:
 * bump to next bugfix release
 * 
 * Version 2.4.9a
 * 16.12.2016:
 * configure additional metadata element for HSA configuration (Ausprägung Einzelstück)
 * update dependency to DocuteamPacker 2.4.9 (extended EAD exporting)
 * EAD export template for CMISTAR
 * 24.11.2016:
 * bump to next bugfix release
 * 
 * Version 2.4.8b
 * 24.11.2016:
 * cleanup resources
 * bump to next bugfix release
 * 
 * Version 2.4.8a
 * 24.11.2016:
 * update dependency to DocuteamPacker 2.4.8
 * extend build from SVN to optionally include/exclude OOO
 * 13.11.2016:
 * add client resources
 * 10.11.2016:
 * bump to next bugfix release
 * 
 * Version 2.4.7b
 * 10.11.2016:
 * fix build to use proper package name
 * bump to next bugfix release
 * 
 * Version 2.4.7a
 * 10.11.2016:
 * update dependency for aipcreatorETH to 1.3.3
 * update dependency for DocuteamPacker to 2.4.7
 * update dependency for docuteam-darc to 2.18.2
 * 28.10.2016:
 * bump to next bugfix release
 * 
 * Version 2.4.5a:
 * 28.10.2016:
 * update dependency for DocuteamPacker to 2.4.5
 * update dependency for aipcreatorETH to 1.3.0
 * update dependency for docuteam-darc to 2.18.1
 * update dependency for docuteam-tools to 1.11.7
 * add missing/fix translations
 * 27.10.2016:
 * lib/DocuteamPacker.jar: final version 2.4.4
 * 		- keep GUI agnostic of AIPCreator implementation details 
 * lib/AIPCreatorETH-1.3.0-snapshot.jar:
 * 		- add submit and checkIngetFeedback methods
 * 		- review mandatory and optional properties
 * lib/docuteam-darc-2.18.0.jar:
 * 	 	- extend interface for ch.docuteam.darc.ingest.AIPCreator to include checkIngestFeedback(String workspaceFolder)
 * update translations and levels.xml
 * 24.10.2016:
 * lib/DocuteamPacker.jar: development version 2.4.4
 * 		- add property to set a external directory for report templates
 * 		- new report for appraisal
 * 		- add option to delete sources after SIP creation
 * 		- performance of workspace table
 * 		- changed package structure
 * lib/docuteam-darc-2.17.0.jar:
 * 		- move submit method to aipcreator
 * 		- additional metadata elements, validators and defaultExpressions
 *
 * Version 2.3.1b:
 * 27.05.2016 (Andreas):
 * lib/DocuteamPacker.jar: final version 2.3.1
 * 		- gui.LauncherView: fix check for existing SIPs in workspace
 * 		- config_dist/levels*.xml, resources/translations/*.properties: add new metadata element 'extentNote'
 * lib/docuteam-darc-2.16.3.jar:
 * 		- mets.Document: add logging information for submit through SFTP
 * 
 * Version 2.3.1a:
 * 24.05.2016 (Andreas):
 * lib/DocuteamPacker.jar: ongoing version 2.3.1
 * - gui.LauncherView, config/docuteamPacker.properties, resources/translations/*: new property "docuteamPacker.defaultChecksumAlgorithm" to set a specific algorithm (default is SHA-512)
 * - lib/docuteam-darc-2.16.3.jar:
 * 		- premis.Object: public setter/getter for default checksum algorithm
 * 
 * Version 2.3.0e:
 * 11.05.2016 (Andreas):
 * lib/DocuteamPacker.jar: final version 2.3.0
 * - gui.LauncherView: fix column titles
 * - gui.SIPView: fix initialization of submit folder
 * lib/docuteam-darc-2.16.2, lib/docuteam-tools-1.11.3: fix closing of SFTP streams
 * 10.05.2016 (Andreas):
 * config/docuteamPacker.properties: new properties for SSH connection
 * lib/DocuteamPacker.jar:
 * - gui.LauncherView:
 * 		- further optimization of SIP list
 * - gui.SIPView:
 * 		- set submit status in parent nodes as well for partial (re-)submits
 * 		- display URL metadata in blue and listen for ctrl-clicks on these cells to open them
 * - gui.SIPView, gui.DeleteFileContentAction: add function to delete the file content of a node
 * - resources/translations/*.properties: adjust translations for submit actions
 * - resources/images/Level*CMI.png: icons to be used in the context of CMISTAR
 * lib/AIPCreatorETH-1.2.0.jar:
 * - ingest.AIPCreatorETH:
 * 		- add DOI to DB table
 * 		- change field "FEEDER_PATH" into "SOURCE_PATH"
 * lib/docuteam-darc-2.16.1.jar:
 * - submit will re-submit parent nodes of selected nodes
 * - ead.DID, ead.ComponentAbstract, config/levels*.xml: new dynamic metadata element 'extentNote'
 * - util.XMLUtil: allow dynamic metadata elements to specify a non-used attribute
 * - mets.structmap.NodeFile: new method to remove file content from submitted nodes
 * - mets.Document: new method to submit using SFTP
 * - mdconfig.LevelMetadataElement, mdconfig.MetadataElement, mets.structmap.NodeAbstract: validate default expressions against metadata validators and allowed values
 * lib/docuteam-converter-1.2.3.jar
 * lib/docuteam-tools-1.11.2.jar:
 * - additional checksum algorithms
 * - updated DROID signatures
 * - SFTP file transfers
 * 
 * Version 2.3.0d:
 * 19.02.2016 (Andreas):
 * lib/DocuteamPacker.jar:
 * 		- gui.LauncherView: optimize table data display
 * 		- gui.SIPView: 
 * 			- move logic for setting the SubmitStatus into darc.mets.structmap.NodeAbstract
 * 			- allow inserting files and creating subfolder for submitted nodes
 * lib/aipcreatorETH-1.1.1.jar: updated version
 * 		- ingest.AIPCreatorETH:
 * 			- adapt to migration keeping original as preservation master and preservation copy
 * 			- allow re-submit for Collection nodes
 * 			- add Rosetta deep link to metadata (additional properties "AIPCreator.DeeplinkPatternCollection" and "AIPCreator.DeeplinkPatternIE")
 * lib/docuteam-darc-2.15.0.jar: updated version
 * 		- merged tags 2.14.4 to 2.14.10 from trunk:
 * 			- ingest.AIPCreatorBasic: new AIPCreator that only checks for mandatory metadata elements and readability of files
 * 			- config_levels/levels_all_*.xml: include additional icon filenames
 * 			- mets.Document: simplify filter for valid SIP folders
 * 			- mets.Document: add method saveAndClose which moves the sip instead of copying it (closing the working copy and setting it into ReadWriteNoFileOps mode) using less resources
 * 			- mets.Document, mdconfig.LevelOfDescription: use SAXReader.read() with a File instead of a String to handle file paths with special characters
 * 			- mets.amdsec.DigiprovWithPremis: new method signatures to create PREMIS events with a specific date
 * 			- mdconfig.MetadataElementValidatorDate*: accept a value of 'NoDate' (and translations if available, e.g. "undatiert")
 * 			- mdconfig.MetadataElementValidator*CH: allow dates to have single digits for month/day components
 * 			- mdconfig.MetadataElementValidatorDateHierarchyRangeCH: fix check for null values in child nodes
 * 			- mdconfig.MetadataElementValidatorDateHierarchyRangeCH: fix to include the "NoDate" string into interpretation
 * 			- mdconfig.MetadataElementValidatorDateHierarchyYear: fix to deal with empty/null values and check against child nodes as well
 * 			- mdconfig.MetadataElementValidatorDateRangeCH: fix to allow dates with single digit days/months
 * lib/docuteam-converter-1.2.1.jar:
 * 		- update for PDF-Tools 4.4
 * lib/docuteam-tools-1.10.9.jar
 * 
 * Version 2.3.0c:
 * 26.09.2015 (Andreas):
 * lib/DocuteamPacker.jar: gui.LauncherView: optimize information collection for table display
 * 
 * Version 2.3.0b:
 * 09.09.2015 (Andreas):
 * lib/aipcreatorETH-1.1.1.jar:
 * 		- ingest.AIPCreatorETH: set submitStatus prospectively in EAD for Rosetta objects
 * lib/DocuteamPacker.jar:
 * 		- gui.LauncherView: add string in list to indicate whether a SIP has been (partially) submitted.
 * 		- gui.SIPView: fix retracting of submitStatus
 * lib/docuteam-converter-1.1.1: Use final tagged version
 * 		- config_dist/migration-config_with-pdf-tools.xml: added for use with pdf-tools
 * 		- config_dist/migration-config.xml: added entries for postscript
 * lib/docuteam-darc-2.15.0.jar:
 * 		- mets.Document:
 * 			- add method hasSubmittedNodes() to check if parts of the SIP have been submitted.
 * 			- properly lock SIPs while being submitted
 * 			- properly unlock SIPs after getting the feedback
 * 		- structMap.nodeFile: fix migration when keeping original, take into account that the SIP can have a submit status blocking regular modifications
 * 		- util.XMLUtil: replace both \r and \n with <EAD:lb> elements
 * lib/docuteam-tools-1.10.8: Use final tagged version
 * 		- file.FileUtil: fix basicCopyTo method to ignore copying if source equals destination
 * lib/docuteam-tools-1.10.7: Use final tagged version
 * 		- lib/log4j-1.2.17.jar: updated version
 * 		- lib/commons-codec-1.9.jar: dependency of poi-3.11-20141221.jar
 * 10.08.2015 (Andreas):
 * lib/DocuteamPacker.jar:
 * 		- gui.SIPView:
 * 			- fix check for SubmitStatus of node (only check nodes that are marked for submission; don't try to change status for already submitted nodes)
 * 			- fix setting of SubmitStatus by first checking both directions in the hierarchy before actually applying the changes.
 * lib/docuteam-darc-2.15.0.jar:
 * 		- mets.structMap.NodeAbstract: split method setSubmitStatusRecursivelyAllOrNone() into two separate ones: ..._check() and _force()
 * 
 * Version 2.3.0a:
 * config/levels.xml: include additional MetadataElement (accessPolicy, accessRestrictionClassification, accessRestrictionPrivacy, accessRestrictionStatus, accessRestrictionStatusExplanation, usagePermission, usagePermissionExpiringDate, sourceType, modeOfAcquisition, relationPeriod, accessionNumber, digitization, biographicalHistory, deathOfAuthor, accruals, characteristicsNote, subject)
 * gui.LauncherView: extend AboutView with JVM information
 * apps/openoffice4.zip: include distribution for linux
 * lib/DocuteamPacker.jar:
 * 		- gui.SIPView:
 * 			- don't allow dragging if the node is being/has been submitted:
 * 			- configure logic for selection of specifics elements for submission in parts
 * 			- automatically set submit status for parent nodes as well
 * 			- enable GUI elements for submission in parts
 * 		- resources/translations/*.properties: adjust translations for submit actions
 * lib/aipcreatorETH-1.1.0.jar:
 * 		- adjust various methods to allow specific selection of nodes to submit
 * lib/docuteam-darc-2.15.0.jar:
 * 		- mets.Document: remove locking of entire SIPs after submit and submit feedback
 * 		- mets.structMap.NodeAbstract: add method to switch a specific SubmitStatus for all nodes from one state to another
 * 		- mets.Document: use the above mentioned new method to prepare the SIP for submission
 * 		- mets.structmap.NodeFile: change behavior when migrating files to create a new subfolder containing original and copy
 *
 * Version 2.2.0a:
 * 20.05.2015 (Andreas):
 * lib/DocuteamPacker.jar: Use final tagged version 2.2.0
 * 		- gui.SIPView: larger input dialogs
 * 		- gui.LauncherView, gui.SIPView: re-worked shortcuts
 * 		- gui.LauncherView: add JVM information into AboutView
 * 		- gui.LauncherView: don't close the application while saving SIPs
 * 		- gui.LauncherView, gui.SIPView: trim text field inputs
 * 		- gui.LauncherView: properly set locale to take effect on default dialogs
 * 		- resources/templates/reports/*: add generic reports
 * lib/aipcreatorETH-1.0.0: Use final tagged version
 * 		- require docuteam-darc-2.14.2
 * lib/docuteam-darc-2.14.2.jar: use tagged library version with
 * 		- additional metadata elements
 * 		- revised handling of PREMIS events
 * 		- resources/templates/mets_sip.xml: remove <metsHdr> to enable initialization of a clean header for new SIPs.
 * 		- mets.amdsec.DigiprovWithPremis: updated PREMIS_Version to 2.2
 * 		- mets.structmap.NodeAbstract: add exception for root file if mimetype or PUID is unknown
 * 		- mets.Document: change xml output format to be compact; saves space, formatting wasn't really indented in a readable way before either.
 * 		- mdconfig.MetadataElementValidatorDateHierarchyRangeCH: ignore children if respective MetadataElement is not available with them
 * 		- mets.Document: collect exception if mets file has no TYPE attribute (i.e. reference for a submission agreement)
 * lib/docuteam-converter-1.1.0: Use final tagged version
 * 		- accept multiple target pronom IDs now (comma separated)
 * 		- use a unique temporary filename as conversion target, then rename it to the correct name
 * 		- do not duplicate the extension even if the output file will have the same name as the input file
 * 		- skip migration if no applicationID is provided
 * 		- config_dist/migration-config.xml: changed ghostscript parameter to -sColorConversionStrategy=/UseDeviceIndependentColor
 * lib/docuteam-tools-1.10.6: Use final tagged version
 * 		- config/*: updated DROID signature files
 * 		- string.StringUtil: new method isDecimal(String)
 * 		- lib/droid*: clean droid libraries
 *
 * Version ETH2.1.3a:
 * 24.02.2015 (Christian):
 * config_dist/migration-config.xml: changed ghostscript parameter to -sColorConversionStrategy=/UseDeviceIndependentColor#
 * 17.02.2015 (Andreas):
 * build.xml: set java compatibility to 1.7
 * config_dist/levels_cmi_de.xml: configuration for the CMISTAR release 2015
 * lib/docuteam-*.jar: use java 1.7 compatible build
 * lib/AIPCreator*.jar: use java 1.7 compatible build
 * lib/droid*.jar: use updated libraries without internal log4j configurations
 * lib/poi*.jar: use updated library
 * 20.11.2014 (Andreas):
 * use docuteam packer 2.1.3 with:
 * 		- gui.LauncherView: show size for non-zipped SIPs
 * 		- gui.FilePreviewer: support preview for JPEG2000
 * 		- gui.SIPView: support new properties:
 * 			- docuteamPacker.SIPView.TreeExpandAll = [true/false]
 * 			- docuteamPacker.SIPView.DefaultTab = [1,2,3]
 * 		- gui.SIPView, gui.LauncherView: support mode 'readWriteNoFileOps' to open SIPs for editing of metadata only (no changes on files/structure).
 * 		- gui.SIPView: fix rendering of the metadata table to select the correct cell when using multiline elements.
 * use aipcreatorETH-1.0.0i with:
 * 		- ingest.AIPCreatorETH: use prepared statements instead of plain text statements to prevent sql injection and do automatic character escaping.
 * 		- ingest.AIPCreatorETH: add workspace label to the file-IE's <dnx:key @id="fileOriginalPath"> (cf. ETH-166)
 * use docuteam-converter-1.0.3 with:
 * 		- use docuteam-tools-1.10.4
 * use docuteam-darc-2.14.1 with:
 * 		- use docuteam-tools-1.10.4
 * 		- common.NodeAbstract, util.XMLUtil: line breaks in metadata elements written as <EAD:lb/> tags now.
 * 		- mets.Document: allows modes readWrite, readWriteNoFileOps and readOnly. When saving a sip in noFileOps mode, just the mets file is saved as backup when using backup.
 * 		- mdconfig.MetadataElementValidatorDateRangeCH: new validator checking a given date range for logic.
 * 		- mdconfig.MetadataElementValidatorDateHierarchyRangeCH: new validator that checks if a date range is within the range of the parent's values and the childrens' date ranges are within the range to validate.
 * 		- mdconfig.MetadataElementDefaultExpressions: new method getUniqueNumericValueWithParentsPrefix(NodeAbstract, MetadataElement) to be used as defaultExpression to create unique values among siblings.
 * 		- mdconfig.MetadataElement: it is possible now to set several validators and postActions per element, separated by semicolons.
 * use docuteam-tools-1.10.4 with:
 * 		- file.FileUtil: add method to get the human readable size of a file
 * 		- util.JavaInterpreter: set method execute(Object, String) as deprecated; changed method execute(Object, Object, String) to method using varargs as: execute(String, Object...)
 * 05.11.2014 (Christian):
 * resources/images adapted icons for mode nofileops
 * 27.10.2014 (Christian)
 * gui.LauncherView: added functionality for new work mode readWriteNoFileOps
 * gui.SIPView: added functionality for new work mode readWriteNoFileOps
 * resources/images: OpenReadWriteNoFileOps.png, OpenInWorkspaceReadWriteNoFileOps.png müssen noch gezeichnet werden.
 * Translations.properties, Translations_fr.properties, Translations_en.properties, Translations_de.properties ergänzt.
 * 13.10.2014 (Christian):
 * docuteamPacker.properties extended by properties docuteamPacker.SIPView.TreeExpandAll(boolean) and docuteamPacker.SIPView.DefaultTab(int [1-3]), code changes in ch.docuteam.packer.gui.SIPView (ETH-161)
 * 02.10.2014 (Christian):
 * postactions.MetadataElementSetterPostActionAddValueInParents: if an empty element of MetadataElementInstance already exist use it to fill in new value; if the value of an element is empty propagate change to parents
 * 17.09.2014 (Christian)
 * postactions.MetadataElementSetterPostActionAddValueInParents: if newValue is null, check siblings and remove value from parent if not found.
 * 
 * Version ETH2.1.2d:
 * 07.09.2014 (Andreas):
 * Use current version of packer 2.1.2 with extension:
 * 		- gui.SIPView:
 * 			- when inserting files or folders, show a message if the submission agreement prohibits some of the files.
 * 			- use hour glass cursor during file replacement.
 * 			- allow multiple selection for context menu (alter change from 31.01.2014).
 * 			- use copy cursor for drag and drop from the file system.
 * 			- use StyledComboBox with variable width of the dropdown list.
 * resources/translations: add additional validator messages
 * licenses/*: general cleanup
 * Use current version of converter 1.0.3 with extension:
 * 		cleanup of libraries and licenses
 * Use current version of darc 2.14.1 with extension:
 * 		mdconfig.*: additional/modified validators
 * 		mdconfig.MetadataElementValidator: return exception instead of string in case of a non-valid value.
 * 		mdconfig.*: new interface for a setterPostAction
 * Use current version of tools 1.10.4 with extension (from 1.10.3):
 * 		new methods for String handling and usage of environment variables
 * 03.09.2014 (Andreas):
 * admin.Version: cleanup comments for next release
 * lib/pdf-renderer-1.0.5.jar: remove unused library
 * templates/reports/Metadaten-Quer.xsl: include report from ETH
 * 07.08.2014 (Christian)
 * resources/translations: merge with the generic sources from packer.
 * 26.06.2014 (Denis):
 * validators.MetadataElementValidatorAddValueInParent: moved to postactions.MetadataElementSetterPostActionAddValueInParents.
 * 		It is now an implementation of MetadataElementSetterPostAction (before: MetadataElementValidator).
 *
 * Version ETH2.1.2c:
 * 12.06.2014 (Denis):
 * Use current version of packer 2.1.2 with extension:
 * 		util.CSVExport: New class.
 * 		gui.SIPView: Correction of the "Packet" menu.
 * Use current version of AIPCreatorETH 1.0.0f.
 * 12.06.2014 (Andreas):
 * resources/templates/reports/*: first publishable version of report
 * 11.06.2014 (Denis):
 * Use current version of packer 2.1.2 with extension:
 * 		util.XML2PDF: Create a report.
 * validators.MetadataElementValidatorAddValueInParent: Renamed (before: MetadataElementValidatorAccessRestrictionPeriod_AddValueInParents)
 * 		Changed so that this validator can be used with any dynamic metadata, not only with "AccessRestrictionPeriod".
 * validators.MetadataElementValidatorAddValueInParent:
 * 		Correction: removed recursion in method check() because it sends setDynamicMetadataValueForName() to the parent
 * 		and is therefore implicity recursive by itself.
 * gui.SIPView: Integrate report creation in SIPView menu.
 * 06.06.2014 (Denis):
 * Use current version of packer 2.1.2 with correction:
 * 		gui.SIPView: On non-empty search results, show in footer text "Elements found: xxx".
 * 05.06.2014 (Denis):
 * Use current version of packer 2.1.2 with correction:
 * 		mets.Document, mets.structmap.NodeAbstract: Improve the searchForAllQuoted() method: ALL search words must be present, but each in any of the searched areas.
 * 		validators.MetadataElementValidatorAccessRestrictionPeriod_AddValueInParents: New class to fill the parent's accessRestrictionPeriod. (Still under development)
 * 04.06.2014 (Denis):
 * gui.SIPView, gui.SearchView: On empty search results, show in footer red text "No Elements found".
 * gui.LauncherView: Display a message box when opening a locked SIP.
 * gui.LauncherView: The column "lastModified" shows the lastModified timestamp of the 'mets.xml' file within, if the SIP is a folder.
 * Use current version of packer 2.1.2 with extension:
 * 		gui.SIPView, gui.SearchView: On empty search results, show in footer red text "No Elements found".
 * 		MetadataProviderDROID: Use full extension matching
 * 		ch.docuteam.darc.mdconfig.MetadataElementValidatorDateCH added. Checks if a date conforms to swiss dateformat.
 * 		ch.docuteam.unittest.darc.mdconfig.MetadataElementValidatorDateCH for unittest.
 * 		ch.docuteam.darc.mdconfig.MetadataElementValidatorUniqueValueAmongSiblings added. Checks if a refCode within same level is unique.
 * 		ch.docuteam.unittest.darc.mdconfig.MetadataElementValidatorUniqueValueAmongSiblings for unittest.
 * 03.06.2014 (Denis):
 * gui.SIPView: ComboBox with dynamic metadata elements which can be added is now alphabetically sorted.
 * Use current version of packer 2.1.2 with extension:
 * 		gui.FilePreviewer: Optional textual preview for files with unknown type.
 *		mdconfig.MetadataElementValidatorUniqueReferenceCode: New validator.
 * 02.06.2014 (Denis):
 * Use current version of packer 2.1.2 with extension:
 * 		config_dist/docuteamPacker.properties, gui.LauncherView:
 * 				New property "docuteamPacker.newSIPDefaultsToZipped". When true (case doesn't matter), the default format of a new SIP is a zipped
 * 				archive; otherwise it is a folder (undefined = true).
 * 29.05.2014 (Andreas):
 * Use current version of AIPCreatorETH 1.0.0f
 * 		includes bugfix to handle non-zipped SIPs
 * 28.05.2014 (Andreas):
 * Use current version of AIPCreatorETH 1.0.0e
 * gui.SIPView: override submit method to push the local levels.xml to the submit directory
 * Use current version of packer 2.1.2 with corrections:
 * 		Use current version of darc 2.13.1 with extensions:
 * 			mets.Document, mets.structmap.NodeAbstract: Extend search to multiple search items, quoted.
 * 			mets.Document, exceptions.ZIPDoesNotContainMETSFileException: When a zipped SIP doesn't contain the METS file at the root level, throw this exception.
 * 		gui.SIPView, gui.SearchView: Extend packet and workspace search to multiple search items, quoted.
 * 		gui.SearchView: Extend workspace search with the possibility to choose between AND- and OR-search.
 * 27.05.2014 (Denis):
 * Use current version of packer 2.1.2 with corrections:
 * 		gui.FilePreviewer: Fixed a bug where in some cases temporary files needed for preview were not deleted under Windows.
 * 23.05.2014 (Denis):
 * Use current version of packer 2.1.2 with corrections:
 * 		mets.Document: Corrections in methods submit(...) and checkIngestFeedback(...): can now handle correctly both zipped and unzipped SIPs.
 * 		gui.FilePreviewer: Correction: Don't delete files needed for preview on packer startup, but only on packer shutdown.
 * 15.05.2014 (Andreas):
 * Use current version of AIPCreatorETH 1.0.0e
 * resources/translations: updated translations
 * 14.05.2014 (Denis):
 * Use current version of packer 2.1.2.
 * gui.LauncherView: Check ingest feedback on startup.
 * Use current version of darc with updated DROID signature file.
 * Use current version of darc with better exception handling when accessing undefined or not configured metadata elements.
 * Use current version of darc with extension:
 * 		new metadata element "accessRestrictionPeriodBaseYear"
 * 		gui.LauncherView: Display Date&Time column in german style (dd.MM.yyyy HH.mm) while column is still sortable
 * Use current version of darc with additional dynamic metadata elements:
 * 		accessPolicy
 * 		accessRestrictionExplanation
 * 		accessRestrictionPeriod
 * 		accessRestrictionClassification
 * 		accessRestrictionPrivacy
 * 		accessRestrictionStatus
 * 		accessRestrictionStatusExplanation
 * gui.LauncherView: Display SIP size in Launcher right-aligned and in kB, not in bytes. SIP list is now sortable.
 * gui.LauncherView: When creating a new SIP (from scratch or from template), reject if a SIP with this name already exists in the folder.
 * gui.SIPView: After inserting nodes, select one of the inserted nodes.
 * 20.02.2014 (Denis):
 * gui.LauncherView$AboutView: Small visual improvements.
 * Use current version of packer 2.1.0 with extensions:
 * 		gui.LauncherView: When creating a copy of a SIP, user can choose if the copy is a zip, or not.
 * 17.02.2014 (Denis):
 * Use current version of tools 1.10.0 with correction:
 * 		gui.SystemOutView: Minor visual improvements in the GUI.
 * 14.02.2014 (Denis):
 * Use current version of packer 2.1.0 with extensions:
 * 		gui.SystemOutView:
 * 				Add "Copy" button to copy the content of the text area into the system clipboard.
 * 				Add method to optionally set an "additional info text" string, which is added to the text area string when copying the text to the clipboard.
 * 		gui.LauncherView: When copying the console text to the system clipboard, include version info, system info, user info, and the current properties from "docuteamPacker.properties".
 * 		gui.LauncherView, translations/Translations*.properties: The URL for the packer help page is now in the translation files.
 * gui.LauncherView$AboutView: Add version info of "docuteam converter".
 * build.xml: Corrections.
 * 10.02.2014 (Denis):
 * Use current version of packer 2.1.0 with extension:
 * 		gui.SIPView: Added action "replaceFileAction".
 * translations/*: Added action "replaceFileAction".
 * 06.02.2014 (Denis):
 * Refactoring: Renamed package "ch.docuteam.docupackETH" to "ch.docuteam.packerETH".
 * Use current version of darc 2.12.0.
 * Use current version of tools 1.10.0.
 * Use current version of AIPCreatorETH 1.10.0d.
 * 04.02.2014 (Denis):
 * translations/*: Remove ISAD-G-Numbers from dynamic metadata element names.
 * Use current version of Docudarc 2.11.4 with corrections:
 * 		mets.Document: Method "checkIngestFeedback()": Proper cleanup after feedback, including removal of the submitted zip file.
 * 03.02.2014 (Denis):
 * Use current version of Docudarc 2.11.4 with corrections:
 * 		mets.Document: Method "checkSubmission()": distinguish between folders and zipped SIPs.
 * 31.01.2014 (Denis):
 * Use current version of Docudarc 2.11.4 with correction:
 * 			mets.Document: Include in "checkSubmission()" the test if the packet already exists in the submission folder.
 * 			Make "checkIngestFeedback()" more strict, and check zipped SIPs as well.
 * Use current version of packer 2.1.0 with correction:
 * gui.SIPView, gui.LauncherView: Right-click first selects the item under the current mouse position, THEN opens the popup menu.
 * 29.01.2014 (Denis):
 * dist_src/docuteam packer.app/Contents/Resources/Info.plist:
 * 		Add Key-Value-Pair "CFBundleExecutable"->"docuteam packer" so that the app can be renamed under OSX.
 * Use current version of packer 2.0.2 with corrections:
 * 		translation/*: Rename the tabs "Information" and "Metadata" to "Administration" and "Description (Beschreibung)"
 * 		gui.SIPView: In the tree view, remove the columns "Children count" and "Descendants count".
 * 		gui.SIPView: In the tree view, select the inserted node.
 * 		dist_src: Refactoring, added linux start script.
 * 24.01.2014 (Denis):
 * Use current version of packer 2.0.2 with corrections:
 * 		gui.SIPView: Fixed bug: in certain occasions it could happen that drag&drop within an SIP didn't work.
 * Use current version of AIPCreatorETH 1.0.0b with corrections:
 * 		AIPCreatorETH: Small improvements in error messaging.
 * 23.01.2014 (Denis):
 * gui.SIPView: The tooltip text of a node now contains additional information whether the file is absent, unreadable or read-only.
 * Use current version of packer 2.0.2 with corrections:
 * 		mets.Document:
 * 				When submitting, zip the SIP without the enclosing folder.
 * 				Made method "boolean isZIPFile()" public.
 * 		gui.LauncherView: HelpPageURL is now up-to-date.
 * Use current version of AIPCreatorETH 1.0.0b.
 * 17.01.2014 (Denis):
 * All java sources: Change copyright message from "2011-2013" to "2011-2014".
 * Use current versions of Docudarc 2.11.4 with correction:
 * 		mets.Document: Correction in revert to the last saved document version: force-unlock the document before reverting.
 * 15.01.2014 (Denis):
 * Use current versions of Docudarc 2.11.4 with correction:
 * 		mets.Document: Correction in revert to the last saved document version: force-unlock the document before reverting.
 * 14.01.2014 (Denis):
 * Use current version of Docudarc 2.11.4
 * Use current version of AIPCreatorETH 1.0.0a.
 * Use Docutools 1.9.1 with extensions:
 * 		gui.GUIUtil: new Class.
 * 		file.PropertyFile: Added new method get(String key, String ifAbsent, String ifEmpty) with additional option "ifEmpty".
 * gui.LauncherView:
 * 		When initializing folders, don't use "project home" as default, but "user home".
 * 		The default for the temp dir is now the folder "DocuteamPacker" in the java temp dir.
 * 		Create empty SIP dir if it doesn't exists.
 * 06.01.2014 (Denis):
 * translations: Correction in error messages.
 * 19.12.2013 (Denis):
 * Use Docupack 2.0.1 with corrections:
 * 		gui.LauncherView, gui.SIPView: Correction in initialization of IngestSubmitDirectory and IngestFeedbackDirectory.
 * 		gui.LauncherView: Refresh views after submit feedback.
 * 13.12.2013 (Denis):
 * resources/translations/Translations*.*: Small corrections.
 * 04.12.2013 (Denis):
 * Use Docudarc 2.11.4 with correction:
 * 		mets.Document: Reset the lock when submit fails. (still under development)
 * 02.12.2013 (Denis):
 * Use Packer 2.0.2.
 * < 02.12.2013 (Denis):
 * Use Docupack 2.0.1 with extensions in AboutView.
 * dist_src/docuteam packer.sh, dist_src/docuteam packer.app/Contents/MacOS/docuteam packer:
 * 		Added 'export LC_CTYPE="UTF-8"' to the scripts to avoid "?" in java file names with other than ascii characters.
 * Use Docupack 2.0.1 with correction:
 * 		gui.FilePreviewer: Handle the case that the OOConverter is not available.
 * Use Docudarc 2.11.2 with correction:
 * 		util.FileConverter: Fast initialization:
 * 		When initializing the OOConverter in a separate thread AND the service is being requested, wait for the initialization to finish.
 * Use Docutools 1.8.4 with extension:
 * 		file.PropertyFile: Resolve environment variables in propertyFile values.
 * Use Docupack 2.0.1 with correction:
 * 		gui.LauncherView: Cleanup initialization() method and accelerate startup by using util.FileConverter's fast initialization.
 * Use Packer 2.0.1:
 * 		With AIPCreatorProxy.
 * ingest.AIPCreatorRosetta: New class (moved from Docutools to here).
 * ingest.AIPCreatorRosetta: Class removed.
 * gui.Launcher: Use class AIPCreatorETH instead of AIPCreatorRosetta.
 * build.xml: New targets 'getAIPCreatorFromLocal' and 'getAIPCreatorFromSVN'.
 * Use Packer 2.0.1 with extension:
 * 		gui.SIPView: The DropDown lists contain now of a List<KeyAndValue> and not any more a List<String>.
 *
 *
 * ETH2.0.0 (Denis, 30.08.2013):
 * resources/translations: Removed unneeded translation files.
 * dist_src: Renamed *.bat to *.cmd.
 * build.xml, dist_src: Added new folder "additionalImages". Copy the additional images from that folder to the folder resources/images on build.
 * build.xml, dist_src: Move build from "Docupack" to "docuteam packer".
 * Use Docupack 2.0.0.
 * gui.SIPView, gui.LauncherView: Override some methods to make sure to deal only with ETH-SIPViews.
 * Created Tag: Version 2.0.0.
 *
 * ETH2.0.0a (Denis, 21.08.2013):
 * Initial version ETH2.0.0a
 * Use Docupack 1.6.2 with Docudarc 2.11.1 and Docutools 1.8.4.
 * Refactoring: Adapt to current version of Docupack.
 * build.xml: Refactoring: Switch to OS X Application package format.
 * Issue ETH-73: Directory Definition für Mac & Windows
 * 		config_dist/docupack.properties: Different file paths for Windows and OSX.
 * Issue ETH-51: Rückfrage bei Export vor Überschreibens
 * 		When exporting files or folders, don't delete possibly existing files in the destination folder (= merge).
 * Issue ETH-85: Anzahl der Versionen beschränken
 * 		config_dist/docupack.properties: Set the max. number of backups; exeeding backups will be deleted.
 * Issue ETH-40: Ungewollte Aufwärtsvererbung
 * Use Docutools 1.8.4 with correction:
 * 		gui.JTableWithSpecificCellEditorPerRow: Added method "commitOrCancelCurrentCellEditor()" to commit (or cancel if commit fails) the current cell editor.
 * Issue ETH-80: Docupack Launcher
 * Use Docupack with corrections:
 * 		gui.LauncherView and gui.SIPView: New classes.
 * build.xml: Don't overwrite config_dist/log4j.xml when getting Docudarc.
 * build.xml: Refactoring: Moved DocupackETH.app and scripts/* into dist_src.
 * Use Docupack with corrections:
 * 		Drag&Drop
 * 		Each document has now its own LevelOfDescriptionSet.
 * 		Redesigned Launcher
 * Issue ETH-84: SIP Daten werden gelöscht
 * Use Docupack with corrections:
 * 		Throw an Exception when saving the SIP while a file or folder inside the SIP is in use, e.g. by another application or the Win7 file explorer.
 * Issue ETH-77: Suche
 * 		Search in SIP
 * 		Search in Workspace (still under development)
 * Use Docupack with corrections:
 * 		gui.FilePreviewer: Fixed a bug when trying to preview a file that is not readable or not existent.
 *
 * 1.0.1 (Denis, 20.12.2012):
 * gui.MainView: Remove all "Template" functionality because it is now implemented in Docupack.
 * Use Docupack 1.6.1 with Docudarc 2.10.1 and Docutools 1.8.0.
 *
 * 1.0.0 (Denis, 13.12.2012):
 * resources/images: Added ETH-specific icons.
 * resources/translations: Added ETH-specific translation files.
 * gui.MainView: The file NextNumber (needed for the DOIGenerator) is now in the individual config-folder.
 * gui.MainView: When opening a SIP, now the file selector allows only folders which contain a mets.xml file.
 * Display translated metadata tooltip texts in the MetadataTable.
 * Ask if to overwrite existing files or folders when exporting.
 * Use Docudarc 2.9.5 with new level property "boolean isAlwaysDisplayed".
 * When creating a template from a SIP, delete all metadata.
 * Use the TAB-Key instead of the Alt-TAB-Key to leave the edit modus and commit the editor changes in the TextAreaTableCellEditor.
 * Use Docudarc 2.9.5 with new level property "boolean keepInTemplate"
 * 		and with deletion of all metadata except those marked with "keepInTemplate='true'" when creating a template from a SIP.
 * Optional label for dynamic metadata comboBox.
 * To calculate the file type for previewing, FIRST check the PUID, THEN the MIME type, and FINALLY the file extension.
 * Subclasses of MainView can now define own version strings.
 * When creating a new node, set the unitTitle to the node's label (= file name), if the node is a folder. If the node is a file, cut away the label's extension.
 * Dynamic Metadata table: To open a ComboBox in the metadata table, double-click it now (before: single-click).
 * Dynamic Metadata table: When the first item of a ComboBox list is a "*", this ComboBox is editable.
 * Initialization of a new subelement's level with the parent level's defaultSubLevel. Before: "Undefined".
 * resources/images: Added new icon "Packet.png" and replaced "MenuFile.png".
 * Improved FileOpenDialog by showing SIPs with a different icon and making SIP folders not traversable (= can not enter).
 * Use Docudarc 2.10.0 with cleanup of temporary files needed for preview.
 *
 * 0.0.6 (Denis, 08.11.2012):
 * PID is now a dynamic metadatum.
 * config_dist/levels.xml: Add PID to MetadataElements.
 * gui.MainView: When changing a node's level, initialize dynamic metadata if necessary and possible.
 * resources/translations/Translations*.properties: Translate PID into DOI.
 * config_dist/levels.xml: Initialize dynamic metadata elements "unitTitleAdditional", "author", "publisher", and "year" with the corresponding values from the root element.
 * gui.MainView: Handle restricted files.
 * gui.MainView, translations/Translations*.properties: Small corrections and cleanup.
 * gui.MainView$TableTransferHandler: Bug fix: importing files and folders using D&D was disabled - it is now re-enabled and respects file access permissions.
 * config_dist/docupack.properties: When setting the folder of the OOO application for converting files, now only the location of the "OpenOffice.org" folder must be provided, not the complete path to the executable file within.
 * gui.MainView: Bug fix: When inserting a new folder, don't restrict to only "safe" characters for the folder name.
 * gui.FilePreviewer: Bring "separate preview" window to the front when clicking on the "Open Separately" button.
 * gui.MainView, translations/Translations*.properties: ToolTipText for dynamic metadata comboBox.
 *
 * 0.0.5 (Denis, 08.10.2012):
 * gui.MainView: Button for access rights.
 * gui.MainView: Show message on open, close, and quit if nodes exist with empty mandatory values.
 * config_dist/levels.xml, config_dist/levels.xsd: Add "AllowedValuesSeparator" to "levels.xml".
 * config_dist/levels.xml, config_dist/levels.xsd: Add "displayRows" to "levels.xml".
 * Use Doculib with class NextNumber.
 * util.DOIGenerator: Added class for generating DOIs
 * config_dist/docupack.properties: Add property "eth.DOIPrefix".
 * resources/images: Added icons "Trash.png", "TrashOK.png", "Trash.NOK.png", "TrashWarning.png", "TrashQuestion.png".
 * config_dist/docupack.properties: Added property "docupack.backupDir". Renamed property "docupack.TemplateDir" to "docupack.templateDir".
 * resources/translations/Translations_*.properties: Added required translations.
 * mets.Document: Correction in the template handling: Unlock the template and cleanup after creating a template.
 *
 * 0.0.4 (Denis, 19.09.2012):
 * Use Docupack 1.5.5: Enable MainView subclassing.
 * gui.MainView: Save SIP as template, New SIP from template.
 * build.xml: Correction in the ant target "getDocupack": Don't get docupack.properties from Docupack because we have additional properties.
 * build.xml: Correction in the ant target "getDocupack": Don't get Translations*.properties from Docupack because we have additional translations.
 *
 * 0.0.3 (Denis, 14.09.2012):
 * Tests for requirement 6f: Auto-fill metadata elements. (suspended)
 * Use Docupack 1.5.4: Integrate all remaining dynamic metadata fields.
 *
 * 0.0.2 (Denis, 22.08.2012):
 * Use Docupack 1.5.3.
 * scripts.*: Added batch-files to start Docupack with different configurations.
 * build.xml: Correction in the ant target "getDocupack": Don't get levels.xml from Docupack because the ETH people use their own levels configuration.
 * build.xml: Correction in the ant target "dist": Copy the batch-files script/* to dist.
 *
 * 0.0.1 (Denis, 08.08.2012):
 * Use Docupack 1.5.2.
 * *.*: Moved everything related to ETH from the project Docupack to here.
 *
 * 0.0.0 (Denis, 07.06.2012):
 * Initialization and setup.
 *
 */
public abstract class Version
{
	static public final String Product = "docuteam packer";
	static public final String Number = "ETH2.4.11a";
	static public final String LastChange = "31.01.2017";

	/**
	 * This main method is being used by the ant build script to be able to add a version number to the compiled jar file.
	 *
	 * @param args: No parameters are expected.
	 */
	public static void main(String[] args)
	{
		System.out.println(Number);
	}
}
