/**
 *	Copyright (C) 2011-2016 Docuteam GmbH
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 3
 *	as published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.docuteam.packerETH.gui.launcher;

import java.awt.event.MouseEvent;
import java.lang.reflect.Method;

import javax.swing.JOptionPane;

import org.apache.commons.cli.CommandLine;

import ch.docuteam.darc.mets.Document;
import ch.docuteam.darc.mets.Document.Mode;
import ch.docuteam.packer.gui.FileProperty;
import ch.docuteam.packer.gui.launcher.Column;
import ch.docuteam.packerETH.gui.launcher.actions.AboutAction;
import ch.docuteam.packerETH.gui.sipView.SIPView;
import ch.docuteam.packerETH.util.DOIGenerator;
import ch.docuteam.tools.os.OSXAdapter;
import ch.docuteam.tools.os.OperatingSystem;
import ch.docuteam.tools.out.Logger;
import ch.docuteam.tools.translations.I18N;


/**
 * @author denis
 *
 */
public class LauncherView extends ch.docuteam.packer.gui.launcher.LauncherView
{
	//	===========================================================================================
	//	========	Structure				=======================================================
	//	===========================================================================================

	//	========	Static Final Public		=======================================================

	//	========	Static Final Private	=======================================================

	//	========	Static Public			=======================================================

	//	========	Static Private			=======================================================

	//	========	Instance Public			=======================================================

	//	========	Instance Private		=======================================================

	//	===========================================================================================
	//	========	Main					=======================================================
	//	===========================================================================================

	protected LauncherView(CommandLine args) {
		super(args);

		aboutAction = new AboutAction(this);
		// Use OSXAdapter to handle cmd-q and the OS X "About" menu entry:
		try {
			OSXAdapter.setQuitHandler(this, getClass().getMethod("requestQuit"));
			Method openAboutWindowMethod = aboutAction.getClass().getMethod("openAboutWindow");
			OSXAdapter.setAboutHandler(this, openAboutWindowMethod);
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * @param args
	 */
	public static void main(String[] args)
	{
		//	Initialize:
		CommandLine commandLine = getCommandLine(args);

		//	Create and open the launcher:
		final LauncherView l = new LauncherView(commandLine);
		initializeETH(commandLine);

		//	Create shudown hook:
		Runtime.getRuntime().addShutdownHook(new Thread(){ @Override public void run() { l.shutdown(); }});

		l.setVisible(true);

		//	Check ingest feedback on startup:
		l.checkIngestFeedback(false);

		//	Open a SIP on startup (if specified in the commandLine):
		l.openSIP(commandLine);
	}

	//	===========================================================================================
	//	========	Methods					=======================================================
	//	===========================================================================================

	//	========	Static Initializer		=======================================================

	//	========	Constructors Public		=======================================================

	//	========	Constructors Private	=======================================================

	//	========	Static Public			=======================================================

	//	--------		Initializing		-------------------------------------------------------

	/**
	 * Initialization specific for this project.
	 */
	static private void initializeETH(CommandLine commandLine)
	{
		//	If a config directory is specified, initialize the DOIGenerator with this path:
		//		- NextNumber
		if (commandLine.hasOption("configDir"))
		{
			//	Initialize DOIGenerator:
			String doiGeneratorFile = commandLine.getOptionValue("configDir") + "/NextNumber";

			Logger.debug("    Init DOIGenerator from: " + doiGeneratorFile);

			DOIGenerator.initialize(doiGeneratorFile);
		}
	}

	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------
	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	========	Static Private			=======================================================

	//	--------		Initializing		-------------------------------------------------------
	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------
	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	========	Instance Public			=======================================================

	//	--------		Initializing		-------------------------------------------------------
	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Interface			-------------------------------------------------------
	//	--------		Actions				-------------------------------------------------------
	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------
	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	========	Instance Private		=======================================================

	//	--------		Initializing		-------------------------------------------------------
	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Interface			-------------------------------------------------------
	//	--------		Actions				-------------------------------------------------------

	/**
	 * Overridden to ask if to really open a locked SIP:
	 */
	@Override
	protected void listSelectionWasClicked(MouseEvent e)
	{
		if (e.getClickCount() == 2)
		{
			//	Double-Click:

			if (Document.isLockedBySomebodyElse(this.selectedSIP.getFile()))
			{
				//	If it is locked, ask if to really open the view (0 means: OK):
				if (JOptionPane.showConfirmDialog(this, I18N.translate("LabelIsLocked") + Document.lockedByWhom(this.selectedSIP.getFile()), I18N.translate("TitleOpenSIP"), JOptionPane.OK_CANCEL_OPTION) != 0)
					return;
			}
		}

		super.listSelectionWasClicked(e);
	}


	/**
	 * Open the SIP denoted by sipPath. If it is locked by anybody except the current user, open readOnly.
	 * If this SIP is already open, bring this view to the front.
	 * If admIdToSelect is not null, select the node with this admId.
	 *
	 * Overridden to open the ETH-SIPView instead of the packer-SIPView.
	 * @param readOnly
	 */
	@Override
	protected void openSIP(FileProperty fileProperty, Mode mode, String admIdToSelect)
	{
		//	Don't allow opening document if it is just being saved:
		if (this.sipsWithSavingInProgress.contains(fileProperty))
		{
			JOptionPane.showMessageDialog(this, I18N.translate("MessageCantOpenSIPWhileBeingSaved"), I18N.translate("TitleOpenSIP"), JOptionPane.INFORMATION_MESSAGE);
			return;
		}

		Logger.getLogger().debug("Opening SIP: " + fileProperty.getFile().getAbsolutePath());

		//	Already open? If yes, bring this view to the front:
		for (ch.docuteam.packer.gui.sipView.SIPView openSIPView: openedSIPViews)
			if (openSIPView.getFileProperty().equals(fileProperty))
			{
				openSIPView.toFront();
				openSIPView.requestFocus();

				openSIPView.selectNode(admIdToSelect);

				return;
			}

		ch.docuteam.packerETH.gui.sipView.SIPView sipView = (SIPView) ch.docuteam.packerETH.gui.sipView.SIPView.open(this, fileProperty, mode, admIdToSelect);

		//	Register this sipView:
		this.openedSIPViews.add(sipView);

		//	Update the sipTable to show that this SIP is now locked by the current user:
		int row = sipTable.getSipTableModel().getRowIndexOfSip(fileProperty);
		// row can be -1 when a sip outside the workspace is open
		if (row > -1) {
			sipTable.getSipTableModel().getSipAtIndex(row).setLockedBy(OperatingSystem.userName());
			sipTable.getSipTableModel().fireTableCellUpdated(row, Column.LOCKED_BY.ordinal());
		}
	}

	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------
	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	===========================================================================================
	//	========	Inner Classes			=======================================================
	//	===========================================================================================


}
