/**
 *	Copyright (C) 2011-2016 Docuteam GmbH
 *
 *	This program is free software: you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License version 3
 *	as published by the Free Software Foundation.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package ch.docuteam.packerETH.gui.sipView;

import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ImageIcon;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;

import ch.docuteam.darc.mdconfig.LevelMetadataElement;
import ch.docuteam.darc.mets.Document;
import ch.docuteam.darc.mets.Document.Mode;
import ch.docuteam.packer.gui.FileProperty;
import ch.docuteam.packer.gui.launcher.LauncherView;
import ch.docuteam.tools.os.SystemProcess;
import ch.docuteam.tools.translations.I18N;


/**
 * @author denis
 *
 */
public class SIPView extends ch.docuteam.packer.gui.sipView.SIPView
{
	//	===========================================================================================
	//	========	Structure				=======================================================
	//	===========================================================================================

	//	========	Static Final Public		=======================================================

	//	========	Static Final Private	=======================================================

	//	========	Static Public			=======================================================

	//	========	Static Private			=======================================================

	//	========	Instance Public			=======================================================

	//	========	Instance Private		=======================================================

	private AbstractAction			grantAction;

	//	===========================================================================================
	//	========	Main					=======================================================
	//	===========================================================================================

	//	===========================================================================================
	//	========	Methods					=======================================================
	//	===========================================================================================

	//	========	Static Initializer		=======================================================

	//	========	Constructors Public		=======================================================

	//	========	Constructors Private	=======================================================

	/**
	 * add the grantAction to the fileMenu.
	 * @param launcherView
	 */
	protected SIPView(LauncherView launcherView)
	{
		super(launcherView);

		//	Actions:

		//	Override closeAction to call THIS closeButtonClicked() method:
		this.closeAction = new AbstractAction(I18N.translate("ButtonClose"), new ImageIcon("./resources/images/Close.png"))
		{	@Override public void actionPerformed(ActionEvent e){ SIPView.this.closeButtonClicked(); }};
		this.closeAction.putValue(Action.ACCELERATOR_KEY, KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.CTRL_DOWN_MASK));
		this.closeAction.putValue(Action.SHORT_DESCRIPTION, I18N.translate("ToolTipClose"));
		this.closeAction.setEnabled(true);

		this.grantAction = new AbstractAction(I18N.translate("ButtonGrant"), new ImageIcon("./resources/images/Grant.png"))
				{	@Override public void actionPerformed(ActionEvent e){ SIPView.this.grantButtonClicked(); }};
		this.grantAction.putValue(Action.SHORT_DESCRIPTION, I18N.translate("ToolTipGrant"));
		this.grantAction.setEnabled(true);

		//	Construct the View:

		//	Menus:

		this.fileMenu.addSeparator();
		this.fileMenu.add(new JMenuItem(this.grantAction));
	}

	//	========	Static Public			=======================================================

	//	--------		Initializing		-------------------------------------------------------
	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------
	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	========	Static Private			=======================================================

	//	--------		Initializing		-------------------------------------------------------

	/**
	 * Open a new SIPView with the SIP passed. If admIdToSelect is not null, select the node with this admId.
	 *
	 * Overridden to use the ETH-SIPView instead of the standard one.
	 * @param zipOrMETSFilePath
	 * @param readOnly
	 * @return
	 */
	static public SIPView open(LauncherView launcherView, FileProperty fileProperty, Mode mode, String admIdToSelect)
	{
		SIPView sipView = new SIPView(launcherView);
		sipView.fileProperty = fileProperty;

		//	Display the window in the desired size:
		if (openFullScreen)
		{
			sipView.setLocation(0, 0);
			sipView.setSize(Toolkit.getDefaultToolkit().getScreenSize());
		}
		else
		{
			sipView.setPreferredSize(new Dimension(screenSizeX, screenSizeY));
			sipView.pack();

			if (screenPosX != null && screenPosY != null)
				sipView.setLocation(screenPosX, screenPosY);
			else
				sipView.setLocationRelativeTo(null);				//	Centered on screen
		}

		sipView.setDividerLocation(sipView.getWidth() / 2);
		sipView.treeTable.requestFocusInWindow();

		sipView.setVisible(true);

		//	Fill the view with contents:
		if (fileProperty.getFile().getName().toLowerCase().endsWith(".zip"))		sipView.read(fileProperty.getFile().getAbsolutePath(), mode, admIdToSelect);
		else											sipView.read(fileProperty.getFile().getAbsolutePath() + "/" + Document.DEFAULT_METS_FILE_NAME, mode, admIdToSelect);

		return sipView;
	}

	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------
	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	========	Instance Public			=======================================================

	//	--------		Initializing		-------------------------------------------------------
	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Interface			-------------------------------------------------------
	//	--------		Actions				-------------------------------------------------------
	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------
	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	========	Instance Private		=======================================================

	//	--------		Initializing		-------------------------------------------------------
	//	--------		Accessing			-------------------------------------------------------
	//	--------		Inquiring			-------------------------------------------------------
	//	--------		Interface			-------------------------------------------------------
	//	--------		Actions				-------------------------------------------------------

	/**
	 * The [Close...] button was clicked.
	 * Returning false means that saving was cancelled.
	 * Overriding standard implementation in order to show message "MessageMandatoryFieldsNotSet" when closing document.
	 */
	@Override
	public boolean closeButtonClicked()
	{
		if (this.document != null && !this.document.isLocked() && this.document.hasNodesWithDynamicMetadataElementInstancesWhichAreMandatoryButNotSet())
			JOptionPane.showMessageDialog(this, "'" + this.document.getSIPName() + "':\n" + I18N.translate("MessageMandatoryFieldsNotSet"), I18N.translate("TitleCloseSIP"), JOptionPane.WARNING_MESSAGE);

		return super.closeButtonClicked();
	}


	private void grantButtonClicked()
	{
		if (this.document == null)		return;

		try
		{
			//	Save document if necessary:
			if (this.document.isModified())		this.document.saveWithoutBackup();

			String originalSIPFolder = this.document.getOriginalSIPFolder();

			//	Open the file browser on the SIP directory:
			if (originalSIPFolder != null)
			{
				if (originalSIPFolder.endsWith(".zip") || originalSIPFolder.endsWith(".ZIP"))
					SystemProcess.openExternally(new File(originalSIPFolder).getParent());
				else
					SystemProcess.openExternally(originalSIPFolder);
			}
			else
					SystemProcess.openExternally(new File(this.document.getFilePath()).getParent());
		}
		catch (java.lang.Exception ex)
		{
			ex.printStackTrace();
		}
	}


	/**
	 * Select in the treeTable the node with the given admId. If this node doesn't exist, ignore it.
	 *
	 * Overridden because of access restriction "protected".
	 * @param admId
	 */
	@Override
	public void selectNode(String admId)
	{
		super.selectNode(admId);
	}

	//	--------		Business Ops		-------------------------------------------------------
	//	--------		Persistence			-------------------------------------------------------
	//	--------		Support				-------------------------------------------------------
	//	--------		Utilities			-------------------------------------------------------

	@Override
	public void enableOrDisableActions()
	{
		super.enableOrDisableActions();

		if (this.getDocument() == null)
		{
			this.grantAction.setEnabled(false);
		}
		else
		{
			this.grantAction.setEnabled(true);
		}
	}


	/**
	 * Overrides the super implementation to sort the combobox items alphabetically.
	 */
	@Override
	protected void fillInsertMetadataElementComboBox()
	{
		List<LevelMetadataElement> sortedLevelMetadataElementsList = this.selectedNode.getDynamicMetadataElementsWhichCanBeAdded();
		Collections.sort(sortedLevelMetadataElementsList,
				new Comparator<LevelMetadataElement>()
				{	@Override
					public int compare(LevelMetadataElement lme1, LevelMetadataElement lme2)
					{
						return I18N.translate(lme1.getId()).compareTo(I18N.translate(lme2.getId()));
					}
				});

		this.selectMetadataElementComboBox.removeAllItems();
		this.selectMetadataElementComboBox.addItem(null);
		for (LevelMetadataElement lme: sortedLevelMetadataElementsList)		this.selectMetadataElementComboBox.addItem(lme);
	}

	//	---------		Misc				-------------------------------------------------------
	//	--------		Debugging			-------------------------------------------------------
	//	---------		Temporary			-------------------------------------------------------

	//	===========================================================================================
	//	========	Inner Classes			=======================================================
	//	===========================================================================================

}
