package ch.eth.docupack.validators;

import java.io.IOException;
import java.util.Properties;

import org.apache.log4j.Logger;

import ch.docuteam.docudarc.mdconfig.MetadataElement;
import ch.docuteam.docudarc.mdconfig.MetadataElementValidator;
import ch.docuteam.docudarc.mets.structmap.NodeAbstract;

/**
 * This class implements a configurable input value length validator.
 * Configuration file has to be named length_validation.conf and has to be
 * placed somewhere on the classpath/validation folder.
 * 
 * @author manuel
 * 
 */
public class LengthValidator implements MetadataElementValidator {

	private final static Logger logger = Logger
			.getLogger(LengthValidator.class);

	static Properties lengthProps = new Properties();

	/**
	 * Try to load length_validation.conf file from $CLASSPATH/validation
	 * folder.
	 */
	static {
		try {
			lengthProps
					.load(ClassLoader
							.getSystemResourceAsStream("validation/length_validation.conf"));
		} catch (IOException e) {
			logger.error("Caught IOException", e);
		} catch (Exception e) {
			logger.error("Caught unexpected Exception", e);
		}
	}

	public String check(String value, NodeAbstract context,
			MetadataElement metadataElement) {

		/* Read meta data element max length from resource properties */
		String sLength = lengthProps.getProperty(metadataElement.getId());

		/*
		 * If there is no entry found for meta data id - nothing has to be
		 * checked
		 */
		if (sLength == null) {
			return null;
		}

		int iLength = Integer.MAX_VALUE;

		try {
			iLength = Integer.parseInt(sLength);
		} catch (NumberFormatException e) {
			logger.error("Caught NumberFormatException parsing [" + sLength
					+ "]");
		}

		return value != null && value.length() > iLength ? "Value too long - maximum "
				+ iLength + " characters."
				: null;
	}

}