<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:EAD="urn:isbn:1-931666-22-9"
	xmlns:xlink="http://www.w3.org/1999/xlink" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:f="Functions" xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:LEVELS="http://www.docuteam.ch/xmlns/levels"
	xsi:schemaLocation="   urn:isbn:1-931666-22-9 http://www.loc.gov/ead/ead.xsd   http://www.w3.org/1999/xlink http://www.loc.gov/standards/mets/xlink.xsd">

	<xsl:output method="xml" encoding="UTF-8" indent="yes" />

	<xsl:variable name="translations" select="unparsed-text('../../translations/Translations.properties', 'ISO-8859-1')" as="xs:string"/>
	<xsl:variable name="lines_translations" as="xs:string*" select="
		for $x in 
			for $i in tokenize($translations, '\n')[matches(., '^[^!#]')] return
				tokenize($i, '=')
			return translate(normalize-space($x), '\', '')" />
	<xsl:function name="f:getTranslation" as="xs:string?">
		<xsl:param name="key" as="xs:string" />
		<xsl:sequence select="$lines_translations[max(index-of($lines_translations, $key))+1]" />
	</xsl:function>
	
	<xsl:variable name="accessors" select="unparsed-text('EAD2Accessor.properties', 'UTF-8')" as="xs:string" />
	<xsl:variable name="lines_accessors" as="xs:string*" select="
		for $x in 
			for $i in tokenize($accessors, '\n')[matches(., '^[^!#]')] return
				tokenize($i, ' = ')
			return translate(normalize-space($x), '\', '')" />
	<xsl:function name="f:getAccessor" as="xs:string?">
		<xsl:param name="key" as="xs:string" />
		<xsl:sequence select="$lines_accessors[index-of($lines_accessors, $key)+1]" />
	</xsl:function>
	
	<xsl:template match="/">
		<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance">
			<fo:layout-master-set>
				<fo:simple-page-master master-name="Deckblatt">
					<fo:region-body margin-top="5cm" margin-bottom="2cm" margin-left="3cm" margin-right="2cm" />
				</fo:simple-page-master>
				<fo:simple-page-master master-name="Folgeseite">
					<fo:region-body margin="2cm" margin-left="3cm" />
				</fo:simple-page-master>
				<fo:page-sequence-master master-name="Bericht">
					<fo:single-page-master-reference master-reference="Deckblatt" />
					<fo:repeatable-page-master-reference master-reference="Folgeseite" />
				</fo:page-sequence-master>
			</fo:layout-master-set>
			<fo:page-sequence master-reference="Bericht">
				<fo:flow flow-name="xsl-region-body" font-size="10pt" page-break-before="always">
					<fo:block-container>
						<fo:block font-weight="bold" font-size="12pt" space-after="6pt">Kassationsbericht</fo:block>
						<xsl:for-each select="//EAD:archdesc">
							<xsl:call-template name="archdesc" />
						</xsl:for-each>
					</fo:block-container>
				</fo:flow>
			</fo:page-sequence>
		</fo:root>
	</xsl:template>
	<xsl:template name="archdesc">
		<xsl:param name="level" select="'0'" />
		<fo:block-container space-after="6pt">
			<!-- <fo:block font-weight="bold" space-after="6pt" keep-with-next="always">
				<xsl:value-of select="@otherlevel" />: <xsl:value-of select="EAD:did/EAD:unittitle" />
			</fo:block>
			<xsl:for-each select="EAD:dsc/EAD:*[not(name(.)='EAD:did') and not(name(.)='EAD:c')]|EAD:did/*[not(name(.)='EAD:unittitle')]">
				<fo:block keep-with-previous="always">
					<xsl:value-of select="concat(replace(name(.), 'EAD:', ''), ': ', .)" />
				</fo:block>
			</xsl:for-each>-->
			<xsl:call-template name="title">
				<xsl:with-param name="level" select="$level" />
			</xsl:call-template>
			<xsl:call-template name="metadata">
				<xsl:with-param name="level" select="$level" />
			</xsl:call-template>
			<xsl:for-each select="EAD:dsc/EAD:c">
				<xsl:call-template name="component" />
			</xsl:for-each>
		</fo:block-container>
	</xsl:template>
	<xsl:template name="component">
		<xsl:param name="level" select="'1'" />
		<fo:block-container space-before="6pt" margin-left="10">
			<xsl:call-template name="title">
				<xsl:with-param name="level" select="$level" />
			</xsl:call-template>
			<xsl:call-template name="metadata">
				<xsl:with-param name="level" select="$level" />
			</xsl:call-template>
			<xsl:for-each select="EAD:c">
				<xsl:call-template name="component">
					<xsl:with-param name="level" select="number($level) + 1" />
				</xsl:call-template>
			</xsl:for-each>
		</fo:block-container>>
	</xsl:template>
	<xsl:template name="title">
		<xsl:param name="level" select="'0'" />
		<fo:block font-weight="bold" keep-with-next="always">
			<fo:table>
				<fo:table-column text-align="left" />
				<fo:table-column column-width="10" />
				<fo:table-column text-align="left" column-width="80%" />
				<fo:table-body>
					<fo:table-row>
						<fo:table-cell>
							<xsl:call-template name="level" />
						</fo:table-cell>
						<fo:table-cell>
							<fo:block />
						</fo:table-cell>
						<fo:table-cell>
							<fo:block>
								<xsl:value-of select="EAD:did/EAD:unittitle" />
							</fo:block>
						</fo:table-cell>
					</fo:table-row>
				</fo:table-body>
			</fo:table>
		</fo:block>
	</xsl:template>
	<xsl:template name="metadata">
		<xsl:param name="level" select="'0'" />
		<fo:block>
			<fo:table width="12cm" space-after="6pt">
				<fo:table-column column-width="3.5cm" />
				<fo:table-column column-width="1cm" />
				<fo:table-column column-width="7.5cm" />
				<fo:table-body>
					<xsl:for-each select="EAD:*[not(name(.)='EAD:dsc' or name(.)='EAD:did' or name(.)='EAD:controlaccess' or name(.)='EAD:c')]|EAD:did/*[not(name(.)='EAD:unittitle')]|EAD:controlaccess/*">
						<fo:table-row>
							<fo:table-cell>
								<fo:block>
									<xsl:if test="count(@*)=0">
										<xsl:variable name="accessorName" select="f:getAccessor(name(.))" />
										<xsl:choose>
											<xsl:when test="$accessorName!=''">
												<xsl:variable name="accessorNameTranslated" select="f:getTranslation($accessorName)" />
												<xsl:choose>
													<xsl:when test="$accessorNameTranslated!=''">
														<xsl:value-of select="$accessorNameTranslated" />
													</xsl:when>
													<xsl:otherwise>
														<xsl:value-of select="concat('[', $accessorName, ']')" />
													</xsl:otherwise>
												</xsl:choose>
											</xsl:when>
											<xsl:otherwise>
												<xsl:value-of select="concat('{', replace(name(.), 'EAD:', ''), '}')" />
											</xsl:otherwise>
										</xsl:choose>
									</xsl:if>
										
									<xsl:for-each select="@*">
										<xsl:if test="not(name(.)='xlink:href')">
											<xsl:variable name="accessorName" select="f:getAccessor(concat(name(..), '[@', name(.), '=''', ., ''']'))" />
											<xsl:choose>
												<xsl:when test="$accessorName!=''">
													<xsl:variable name="accessorNameTranslated" select="f:getTranslation($accessorName)" />
													<xsl:choose>
														<xsl:when test="$accessorNameTranslated!=''">
															<xsl:value-of select="$accessorNameTranslated" />
														</xsl:when>
														<xsl:otherwise>
															<xsl:value-of select="concat('[', $accessorName, ']')" />
														</xsl:otherwise>
													</xsl:choose>
												</xsl:when>
											</xsl:choose>
										</xsl:if>
									</xsl:for-each>
								</fo:block>
							</fo:table-cell>
							<fo:table-cell>
								<fo:block />
							</fo:table-cell>
							<fo:table-cell>
								<fo:block>
									<xsl:choose>
										<xsl:when test="name(.)='EAD:dao'">
											<xsl:value-of select="@xlink:href" />
										</xsl:when>
										<xsl:otherwise>
											<xsl:value-of select="." />
										</xsl:otherwise>
									</xsl:choose>
								</fo:block>
							</fo:table-cell>
						</fo:table-row>
					</xsl:for-each>
				</fo:table-body>
			</fo:table>
		</fo:block>
	</xsl:template>
	<xsl:template name="level">
		<xsl:variable name="currentlevel" select="@otherlevel" />
		<xsl:variable name="image" select="document('../../../config/levels_reports.xml', document(''))//LEVELS:Level[@nameID=$currentlevel]/@iconFileName" />
		<fo:block>
			<!-- <xsl:value-of select="$image"></xsl:value-of> -->
			<xsl:element name="fo:external-graphic">
				<xsl:attribute name="content-height">scale-to-fit</xsl:attribute>
				<xsl:attribute name="height">16px</xsl:attribute>
				<xsl:attribute name="width">16px</xsl:attribute>
				<xsl:attribute name="src" select="$image" />
			</xsl:element>
			<xsl:value-of select="concat(' ', @otherlevel)" />
		</fo:block>
	</xsl:template>
</xsl:stylesheet>
